package com.jpattern.gwt.client.presenter;

import java.util.List;
import java.util.Map;

import com.google.gwt.user.client.ui.HasWidgets;
import com.jpattern.gwt.client.navigationevent.INavigationEvent;
import com.jpattern.gwt.client.navigationevent.INavigationEventCallback;
import com.jpattern.gwt.client.navigationevent.INavigationEventData;
import com.jpattern.gwt.client.navigationevent.INavigationEventWrapper;
import com.jpattern.gwt.client.view.IView;
import com.jpattern.shared.result.IErrorMessage;

/**
 * 
 * @author Francesco Cina'
 *
 * 20 Apr 2011
 */
public interface IPresenter {

	/**
	 * Return the navigationEvent which generated the presenter
	 * @return
	 */
	INavigationEvent getNavigationEvent();
	
	/**
	 * Return whether the presenter ended is initialisation and has been rendered
	 * @return
	 */
	boolean isInitialized();
	
	/**
	 * Clear the error area before the execution of a new event
	 */
	void onEventStart();
	
	/**
	 * Called by the IEvent when the result of the execution is not valid
	 * @param errorMessages
	 */
	void onEventError(List<IErrorMessage> errorMessages);
	
	/**
	 * Bind the presenter's view with his container
	 */
	void render();
	
	/**
	 * Initialize the presenter and show the related IView
	 */
	void initPresenter();

	/**
	 * Set the parent presenter
	 * @param parentPresenter
	 */
	void setParent(IPresenter parentPresenter);
	
	/**
	 * Return an ordered list of all the presenter names in the current hierarchy, from the root parent to the leaf
	 * @return
	 */
	void hierarchy(List<String> hierarchyResult);

	/**
	 * Return the IView object associated with this presenter
	 * @return
	 */
	IView getView();
	
	/**
	 * Register a new navigation event throwable by the current presenter
	 * @param navigationEvent
	 * @param eventTarget the target area where the new view will be rendered
	 * @param navigationEventCallback
	 */
	void registerNavigationEvent(INavigationEvent navigationEvent, HasWidgets eventTarget, INavigationEventCallback navigationEventCallback);
	
	/**
	 * Register a new navigation event throwable by the current presenter
	 * @param navigationEvent
	 * @param eventTarget the target area where the new view will be rendered
	 * @param navigationEventCallback
	 */
	void registerNavigationEvent(INavigationEventWrapper navigationEventWrapper, HasWidgets eventTarget, INavigationEventCallback navigationEventCallback);
	
	/**
	 * Set an event to be executed when the presenter is initialized
	 * @param navigationEventName
	 */
	void addInitialNavigationEvent(String navigationEventName);
	
	/**
	 * Set an event to be executed when the presenter is initialized
	 * @param navigationEventName
	 * @param queryStringValues a map with key value pair to be passed at the navigationEvent exec() method to build the new Presenter
	 */
	void addInitialNavigationEvent(String navigationEventName, Map<String, String> queryStringValues);
	
	
	/**
	 * 
	 * @param navigationEventName
	 */
	void ignoreInitialEventWithSameTargetOf(String navigationEventName);

	/**
	 * Launch a navigation event based on his name.
	 * If an event with that name is present in the current Presenter is thrown, otherwise the event is propagated to the parent presenter 
	 * @param registerHistory indicate if the event must be registered in the navigation history, this must be true only if the event has been thrown by an explicit user action
	 * @param navigationEventName
	 */
	void launchNavigationEvent(String navigationEventName, boolean registerHistory);
	
	/**
	 * Launch a navigation event based on his name.
	 * If an event with that name is present in the current Presenter is thrown, otherwise the event is propagated to the parent presenter 
	 * @param queryStringValues a map with key value pair to be passed at the navigationEvent exec() method to build the new Presenter
	 * @param registerHistory indicate if the event must be registered in the navigation history, this must be true only if the event has been thrown by an explicit user action
	 * @param navigationEventName
	 */
	void launchNavigationEvent(String navigationEventName, boolean registerHistory, Map<String, String> queryStringValues);
	
	/**
	 * Return information related to a specific INavigationEvent
	 * @return
	 */
	INavigationEventData getNavigationEventData(String navigationEventName);

	/**
	 * Return the presenter's name. It correspond to the name of the NavigationEvent which created the presenter.
	 * @return
	 */
	String getName();
	
	/**
	 * This method is called when the view related to this Presenter is destroyed.
	 * In fact this is a real destructor for the presenter.
	 */
	void onOnloadPresenter();
}
